import unittest

from plio.examples import get_path
from plio.io import io_edr


# class Test_Tes_IO(unittest.TestCase):
#
#     # Need different test data or need to modify the current code
#     def setUp(self):
#         self.examplefile = get_path('cl5_398736801edr_f0030004ccam01014m1.dat')
#     #
#     def test_open(self):
#         ds = io_edr.EDR(self.examplefile)
#
# if __name__ == '__main__':
#     unittest.main()
